%**************************************************************************
% This file is part of the MLS-C Library; you can redistribute it and/or 
% modify it under the terms of the GNU Lesser General Public License as 
% published by the Free Software Foundation; either version 3 of the 
% License,or any later version.
% This program is distributed in the hope that it will be useful but 
% WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
% Public License for more details.
% You should have received a copy of the GNU Lesser General Public License
% along with GNU; see the file COPYING.  If not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
% or see <http://www.gnu.org/licenses/>.
%**************************************************************************
% Created by: 
%   E Nadal, M Ouallal, JJ Rodenas, F Chinesta and FJ Fuenmayor
%  
% Release 1.0  
% Date: 18/07/2014
%**************************************************************************
%%
% This Matlab(c) subroutine calls the Moving Least Squares function to 
% evaluate a recovered stress field from a 2D stress field evaluated using 
% the finite element method for linear mesh.

% Remove all blanck lines and headers from the output files (Element 
% solution, Topology, Node Coordinates) generated by Ansys to be used as 
% input argument when calling the MLS subroutine.

AnsysRelease = 14; %Valid releases are 12 and 14

%% Read NLIST.txt   Nodal information
f = fopen('NLIST.txt', 'rt');
if AnsysRelease ==14
    c = textscan(f,'%s','Delimiter','%s','headerlines',4,...
        'CollectOutput',true); %Read file
    lines = c{1};              %Store nodal coords
elseif AnsysRelease == 12
    c = textscan(f,'%s','Delimiter','%s','headerlines',3,...
        'CollectOutput',true); %Read file
    lines = c{1};              %Store nodal coords
    lines(1:21:end) = [];      %Remove blank lines
else
    err ('Ansys release not supported');
end 
fclose(f);

coordinatesXY = zeros(numel(lines),4);

for i = 1:numel(lines)
    coordinatesXY(i,:) = coordinatesXY(i,:) + str2num(lines{i});

end

coordinatesXY(:,[1,4]) = [];
Xcoord                 = coordinatesXY(:,1);
Ycoord                 = coordinatesXY(:,2);

%% Read ELIST.txt   Element information
f = fopen('ELIST.txt','rt');
if AnsysRelease ==14
    c = textscan(f,'%s','Delimiter','%s','headerlines',4,...
        'CollectOutput',true);
    lines = c{1};
elseif AnsysRelease == 12
    c = textscan(f,'%s','Delimiter','%s','headerlines',2,...
        'CollectOutput',true);
    lines = c{1};              
    lines(1:21:end) = [];      
else
    err ('Ansys release not supported');
end 
fclose(f);
 
topology = zeros(numel(lines),10);

for i = 1:numel(lines)
    topology(i,:) = topology(i,:) + str2num(lines{i});

end

topology(:,1:6) = [];

%

f = fopen('PRESOL.txt','rt');
% Ansys 14,12
c = textscan(f,'%s','Delimiter','%s','headerlines',2,...
    'CollectOutput',true); 

lines          = c{1};
lines(1:6:end) = [];
lines(1:5:end) = [];
GaussStresses  = zeros(numel(lines),7);
NumGP          = 4;
NumElem        = size(topology,1);

for i = 1:numel(lines)
    GaussStresses(i,:) = GaussStresses(i,:) + str2num(lines{i});

end

GaussStresses(:,[4,6,7]) = [];
Temporary_Linear = mat2cell(GaussStresses,NumGP*ones(NumElem,1),NumGP);

for i = 1:NumElem
Temp = Temporary_Linear{i,1};
    for k = 2:NumGP
        if Temp(k,1)==Temp(k-1,1)
            Temp(k-1,:)=[];
        end
    end
Temporary_Linear{i,1}=Temp;
end

GaussStresses      = cell2mat(Temporary_Linear);
NodeNumLinear = GaussStresses(:,1);

fclose(f);

%%
% Ansys does not dispose of commands which calculate the cordinate of the
% integration points in the global cordinate system, to do so, this part of
% the code was implemented to obtain the coordinates of these points in the
% global system for quadrilateral, triangle and mixed mesh.


% Number of sides per element
NSidesElemTri    = 3; 
NSidesElemQuad   = 4;
  
% Number of GPs per element
NumPointsQuad    = 4; 
NumPointsTri     = 3;
Dimensions       = 2;
Degree           = 1;
CaldN            = 0;

% Invoke pgauss function to obtain the GP coordinates in the local system 
% reference.
CoordinatesTri  = pgauss(Dimensions,NSidesElemTri,NumPointsTri);
CoordinatesQuad = pgauss(Dimensions,NSidesElemQuad,NumPointsQuad);

% Shape function for triangular and quadrilateral elements.
NTri  = shape_f_2d(CoordinatesTri ,Degree,NSidesElemTri ,CaldN);
NQuad = shape_f_2d(CoordinatesQuad,Degree,NSidesElemQuad,CaldN);

NumElem       = size(topology,1); 
NnodeElem     = size(topology,2); 

% Proceed with calculation of GP coordinates in global system reference.

% GP for quadrilateral elements
if size(GaussStresses,1)==NumElem*NumPointsQuad 
    XQuad = zeros(NumElem,NnodeElem);
    YQuad = zeros(NumElem,NnodeElem);

    for i=1:NumElem
        for j=1:NnodeElem
            XQuad(i,j) = Xcoord(topology(i,j));
            YQuad(i,j) = Ycoord(topology(i,j));
        end
    end

    
    GPXQuad = XQuad*NQuad;
    GPYQuad = YQuad*NQuad;
    TempX   = mat2cell(GPXQuad,1*ones(length(topology),1),NumPointsQuad);
    TempY   = mat2cell(GPYQuad,1*ones(length(topology),1),NumPointsQuad);
    TempX   = cellfun(@transpose,TempX,'un',0);
    TempY   = cellfun(@transpose,TempY,'un',0);
    GaussPtCoordXQuad = cell2mat(TempX);
    GaussPtCoordYQuad = cell2mat(TempY);
    GaussPtCoordXY    = [GaussPtCoordXQuad GaussPtCoordYQuad];
    
    
% GP for quadrilateral and triangular elements    
elseif size(GaussStresses,1)< NumElem*NumPointsQuad &&...
       size(GaussStresses,1)> NumElem*NumPointsTri  
    X = zeros(NumElem,NnodeElem);
    Y = zeros(NumElem,NnodeElem);
    for i=1:NumElem
        for j=1:NnodeElem
            X(i,j) = Xcoord(topology(i,j));
            Y(i,j) = Ycoord(topology(i,j));
        end
    end
    Temporary1 = mat2cell(X,1*ones(NumElem,1),NnodeElem);
    Temporary2 = mat2cell(Y,1*ones(NumElem,1),NnodeElem);
    T1         = zeros(NumElem,NnodeElem);
    T2         = zeros(NumElem,NnodeElem);
    X_tri      = zeros(NumElem,NnodeElem);
    Y_tri      = zeros(NumElem,NnodeElem);
    for k = 1:NumElem
        Temp1 = Temporary1{k};
        Temp2 = Temporary2{k};
        Tg = unique(topology(k,:)','rows');
        if size(Tg',2)==3
           X_tri(k,:)      = X(topology(k,:));
           Y_tri(k,:)      = Y(topology(k,:));
           T1              = X_tri(k,:);
           X_tri(k,:)      = X(k,:);
           Temporary1{k,1} = zeros(1,NnodeElem);
           Temporary2{k,1} = zeros(1,NnodeElem);
           T2              = Y_tri(k,:);
           Y_tri(k,:)      = Y(k,:);
        end 
    end
    
    remCol1    = unique(X_tri','rows','stable');
    XtriCoord  = remCol1';
    remCol2    = unique(Y_tri','rows','stable');
    YtriCoord  = remCol2';
    GPX_tri    = XtriCoord*NTri;
    GPY_tri    = YtriCoord*NTri; 
    XquadCoord = cell2mat(Temporary1);
    YquadCoord = cell2mat(Temporary2);
    GPXquadr   = XquadCoord*NQuad;
    GPYquadr   = YquadCoord*NQuad;
    GPXtri     = [GPX_tri zeros(NumElem,1)];
    GPYtri     = [GPY_tri zeros(NumElem,1)];
    
    for k = 1:NumElem
        for m = 1:NnodeElem
            if GPXquadr(k,:)==0    
                GPXquadr(k,:) = GPXtri(k,:);
            end
            if GPYquadr(k,:)==0 
                GPYquadr(k,:) = GPYtri(k,:);
            end
        end
    end
    
    TempX = mat2cell(GPXquadr,1*ones(length(topology),1),NumPointsQuad);
    TempY = mat2cell(GPYquadr,1*ones(length(topology),1),NumPointsQuad);
    TempX = cellfun(@transpose,TempX,'un',0);
    TempY = cellfun(@transpose,TempY,'un',0);
    GaussPtCoordX  = cell2mat(TempX);
    GaussPtCoordY  = cell2mat(TempY);
    GaussPtCoordX(all(GaussPtCoordX==0,2),:) = [];
    GaussPtCoordY(all(GaussPtCoordY==0,2),:) = [];
    GaussPtCoordXY = [GaussPtCoordX GaussPtCoordY];
    
    
% GP for triangular elements            
elseif  size(GaussStresses,1)==NumElem*NumPointsTri
    Temp         = topology';
    Temp2        = unique(Temp,'stable','rows');
    topologyTemp = Temp2';
    NumElemTri   = size(topologyTemp,1);
    NnodeElemTri = size(topologyTemp,2);
    XTri         = zeros(NumElemTri,NnodeElemTri);
    YTri         = zeros(NumElemTri,NnodeElemTri);
           
    for i=1:NumElemTri
        for j=1:NnodeElemTri
            XTri(i,j) = Xcoord(topologyTemp(i,j));
            YTri(i,j) = Ycoord(topologyTemp(i,j));
        end
    end
            
    GPXTri = XTri*NTri;
    GPYTri = YTri*NTri;
    TempX  = mat2cell(GPXTri,1*ones(length(topologyTemp),1),NumPointsTri);
    TempY  = mat2cell(GPYTri,1*ones(length(topologyTemp),1),NumPointsTri);
    TempX  = cellfun(@transpose,TempX,'un',0);
    TempY  = cellfun(@transpose,TempY,'un',0);
    GaussPtCoordX  = cell2mat(TempX);
    GaussPtCoordY  = cell2mat(TempY);
    GaussPtCoordXY = [GaussPtCoordX GaussPtCoordY];
    
end

GaussStresses(:,1) = [];
% Matrix containing stresses and node coordinates.
FinalData = [GaussPtCoordXY GaussStresses];

%% 
% This function load the data for the MLS smoothing using linear elements

% Define the path where the data is available

FieldFEGP   = FinalData(:,3:5);        % FE stresses
XYZGP       = FinalData(:,1:2);        % Nodes coordinates

NumGP      = size(XYZGP,1);            % Number of Gauss Points GP
NumNodes   = size(coordinatesXY,1);    % number of nodes
XYZout     = [ XYZGP ; coordinatesXY]; % Number of nodes and GP

% Invoke the MLS function for linear element to obtain the recovered 
% stress field at integration point and nodes.
[FieldStGP_Linear]    = MLS(FieldFEGP,XYZGP,XYZout,Mesh);
FieldStGP_Linear_GP   = FieldStGP_Linear(1:NumGP,:);
FieldStGP_Linear_Node = FieldStGP_Linear(1+NumGP:end,:);

%% Prepare data to create to invoke the CreateMacroToGetMLSdata to generate 
% the macro file MLSresults.mac
Linear_SigX  = zeros(size(topology,1),size(topology,2));
Linear_SigY  = zeros(size(topology,1),size(topology,2));
Linear_SigXY = zeros(size(topology,1),size(topology,2));

for i = 1:size(topology,1)
    for j = 1:size(topology,2)
        Linear_SigX(i,j)  = FieldStGP_Linear_Node(topology(i,j),1);
        Linear_SigY(i,j)  = FieldStGP_Linear_Node(topology(i,j),2);
        Linear_SigXY(i,j) = FieldStGP_Linear_Node(topology(i,j),3);
    end
end

Linear_SigZ  = zeros(size(topology,1),size(topology,2));
Linear_SigXZ = zeros(size(topology,1),size(topology,2));
Linear_SigYZ = zeros(size(topology,1),size(topology,2));

% Prepare the input argument for the CreateMacroToGetMLSdata function
OutputXYZ      = [coordinatesXY zeros(length(coordinatesXY),1)];
OutputTopology = unique(topology(:,1:4));
ShowUX         = zeros(length(OutputTopology),1);
ShowVY         = zeros(length(OutputTopology),1);
ShowSX         = Linear_SigX(:,1:4);
ShowSY         = Linear_SigY(:,1:4);
ShowTXY        = Linear_SigXY(:,1:4);
ShowSZ         = Linear_SigZ(:,1:4);
ShowTXZ        = Linear_SigXZ(:,1:4);
ShowTYZ        = Linear_SigYZ(:,1:4);

CreateMacroToGetMLSdata(ShowUX,ShowVY,OutputXYZ,topology(:,1:4),ShowSX,ShowSY,ShowSZ,ShowTXY,ShowTXZ,ShowTYZ)
