%**************************************************************************
% This file is part of the MLS-C Library; you can redistribute it and/or 
% modify it under the terms of the GNU Lesser General Public License as 
% published by the Free Software Foundation; either version 3 of the 
% License,or any later version.
% This program is distributed in the hope that it will be useful but 
% WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
% Public License for more details.
% You should have received a copy of the GNU Lesser General Public License
% along with GNU; see the file COPYING.  If not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
% or see <http://www.gnu.org/licenses/>.
%**************************************************************************
% Created by: 
%    E Nadal, M Ouallal, JJ Rodenas, F Chinesta and FJ Fuenmayor
%  
% Release 1.0  
% Date: 18/07/2014
%**************************************************************************
%%
function [Coordinates,Weights]=...
                       pgauss(Dimensions,NumSideElem,NumPoints,PointClass)
% CALCULATION OF GAUSS POINTS COORDIANTES AND WEIGHTS FOR 
% VERTEX TRIANGLES (0,0),(1,0),(0,1)
% AND VERTEX QUADRILATERAL (-1,-1),(-1,1), (1,1) (1,-1)
%
%
% INPUT
    % Dimensions   1 Countour integral / 2 Integral area
    % NumSideElem  Number of side per element
    % NumPoints    Number of total integration points per element 
    % PointClass   Set of integration points which being taken. 
    %              Used to diferentiate quadrature rules with
    %              equal number of integration points
%     
% OUTPUT
    % Coordinates  Integration points coordinates in the local system.
    % Weights      Weigth of each inegration point.
%==========================================================================
if exist('PointClass')==0
    PointClass=1;
end

%CONTOUR INTEGRAL
%================
if Dimensions==1 
   switch NumPoints
       
   % Polynomial exact integration of degree P=1   
   %-------------------------------------------
   case 1 
      Coordinates = 0;
      Weights     = 2;
   
   % Polynomial exact integration of degree P=3   
   %-------------------------------------------   
   case 2
      Coordinates = [...
                    -0.577350269189626;...
                     0.577350269189626];
      Weights     = [1,1]';
      
   % Polynomial exact integration of degree P=5   
   %-------------------------------------------   
   case 3
      Coordinates  =[...
                    -0.774596669241483;...
                     0;...
                     0.774596669241483];
      Weights      =[...
                     0.55555555555555555;...
                     0.88888888888888888;...
                     0.55555555555555555];
     
   % Polynomial exact integration of degree P=7   
   %-------------------------------------------              
   case 4 
      Coordinates  =[...
                    -0.861136311594953;...
                    -0.339981043584856;...
                     0.339981043584856;...
                     0.861136311594953];
      Weights      =[...
                     0.347854845137454;...
                     0.652145154862546;...
                     0.652145154862546;...
                     0.347854845137454];
      
   % Polynomial exact integration of degree P=9   
   %-------------------------------------------              
   case 5 
      Coordinates   =[...
                     -0.906179845938664;...
                     -0.538469310105683;...
                      0;...
                      0.538469310105683;...
                      0.906179845938664];
      Weights       =[...
                      0.236926885056189;...
                      0.478628670499366;...
                      0.568888888888889;...
                      0.478628670499366;...
                      0.236926885056189];
      
   % Polynomial exact integration of degree P=11   
   %--------------------------------------------               
   case 6 
      Coordinates   =[...
                     -0.932469514203152;...
                     -0.661209386466265;...
                     -0.238619186083197;...
                     +0.238619186083197;...
                     +0.661209386466265;...
                     +0.932469514203152];
      Weights       =[...
                      0.171324492379170;...
                      0.360761573048139;...
                      0.467913934572691;...
                      0.467913934572691;...
                      0.360761573048139;...
                      0.171324492379170;];
      
   otherwise
      disp (['Not implemented for a given dimension '...
            'NumPoints = ',num2str(NumPoints)]);
      error('Stopping');
   end
   
%=========================================================================   
% AREA INTEGRAL
% =============
else  %Dimensions = 2 : 
   switch NumSideElem
   %----------------------------------------------------------------------
   case 3 %Area Integral for triangles
      switch NumPoints
      case 3
         if PointClass==1 %3 points inside the element
            Coordinates=[...
                  1/6,1/6;...
                  2/3,1/6;...
                  1/6,2/3];
            Weights=[...
                  1/6;...
                  1/6;...
                  1/6];
         else             % 3 points in midside.
            Coordinates=[...
                  0.5,0;...
                  0.5,0.5;...
                  0,0.5];
            Weights=[...
                  1/6;...
                  1/6;...
                  1/6];
         end
      case 4
         Coordinates=[...
               1/3,1/3;...
               1/5,1/5;...
               3/5,1/5;...
               1/5,3/5];
         Weights=[...
               -27/96;...
               25/96;...
               25/96;...
               25/96];
      case 6
         if PointClass==1 %6 points inside the elem.
            Coordinates=[...
                  0.091576213509771, 0.091576213509771;...
                  0.816847572980459, 0.091576213509771;...
                  0.091576213509771, 0.816847572980459;...
                  0.445948490915965, 0.445948490915965;...
                  0.108103018168070, 0.445948490915965;...
                  0.445948490915965, 0.108103018168070];
            Weights=[...
                  0.109951743655322/2.0;...
                  0.109951743655322/2.0;...
                  0.109951743655322/2.0;...
                  0.223381589678011/2.0;...
                  0.223381589678011/2.0;...
                  0.223381589678011/2.0];
         else             %6 points in midside.
            Coordinates=[...
                  1/6,1/6;...
                  2/3,1/3;...
                  1/6,2/3;...
                  0.5,0;...
                  0.5,0.5;...
                  0,0.5];
            Weights=[...
                  1/6;...
                  1/6;...
                  1/6;...
                  1/6;...
                  1/6;...
                  1/6];...
         end
      otherwise
         disp (['Not implemented for triangular area '...
               'NumPoints = ',num2str(NumPoints)]);
         error('Stopping');
      end
      
   %-----------------------------------------------------------------   
   case 4 % Area integral for quadrilaterals.
      switch NumPoints
      case 1
         Coordinates=[0,0];
         Weights=4;
         
      case 4
         Coordinates=[...
               -0.57735026918963, -0.57735026918963;...
               +0.57735026918963, -0.57735026918963;...
               -0.57735026918963, +0.57735026918963;...
               +0.57735026918963  +0.57735026918963];
         Weights=[...
               1;...
               1;...
               1;...
               1];
         
      case 9
         Coordinates=[...
               -0.77459666924148, -0.77459666924148;...
               +0               , -0.77459666924148;...
               +0.77459666924148, -0.77459666924148;...
               -0.77459666924148, +0               ;...
               +0               , +0               ;...
               +0.77459666924148, +0               ;...
               -0.77459666924148, +0.77459666924148;...
               +0               , +0.77459666924148;...
               +0.77459666924148, +0.77459666924148];
         Weights=[...
               0.30864197530864;...
               0.49382716049383;...
               0.30864197530864;...
               0.49382716049383;...
               0.79012345679012;...
               0.49382716049383;...
               0.30864197530864;...
               0.49382716049383;...
               0.30864197530864];
         
      case 16
         Coordinates=[...
               -0.86113631159495, -0.86113631159495;...
               -0.33998104358486, -0.86113631159495;...
               +0.33998104358486, -0.86113631159495;...
               +0.86113631159495, -0.86113631159495;...
               -0.86113631159495, -0.33998104358486;...
               -0.33998104358486, -0.33998104358486;...
               +0.33998104358486, -0.33998104358486;...
               +0.86113631159495, -0.33998104358486;...
               -0.86113631159495, +0.33998104358486;...
               -0.33998104358486, +0.33998104358486;...
               +0.33998104358486, +0.33998104358486;...
               +0.86113631159495, +0.33998104358486;...
               -0.86113631159495, +0.86113631159495;...
               -0.33998104358486, +0.86113631159495;...
               +0.33998104358486, +0.86113631159495;...
               +0.86113631159495, +0.86113631159495];
         Weights=[...
               0.12100299328560;...
               0.22685185185185;...
               0.22685185185185;...
               0.12100299328560;...
               0.22685185185185;...
               0.42529330301069;...
               0.42529330301069;...
               0.22685185185185;...
               0.22685185185185;...
               0.42529330301069;...
               0.42529330301069;...
               0.22685185185185;...
               0.12100299328560;...
               0.22685185185185;...
               0.22685185185185;...
               0.12100299328560];
         
      case 25
         Coordinates=[...
               -0.90617984593866,  -0.90617984593866;...
               -0.53846931010568,  -0.90617984593866;...
               +0               ,  -0.90617984593866;...
               +0.53846931010568,  -0.90617984593866;...
               +0.90617984593866,  -0.90617984593866;...
               -0.90617984593866,  -0.53846931010568;...
               -0.53846931010568,  -0.53846931010568;...
               +0               ,  -0.53846931010568;...
               +0.53846931010568,  -0.53846931010568;...
               +0.90617984593866,  -0.53846931010568;...
               -0.90617984593866,  +0               ;...
               -0.53846931010568,  +0               ;...
               +0               ,  +0               ;...
               +0.53846931010568,  +0               ;...
               +0.90617984593866,  +0               ;...
               -0.90617984593866,  +0.53846931010568;...
               -0.53846931010568,  +0.53846931010568;...
               +0               ,  +0.53846931010568;...
               +0.53846931010568,  +0.53846931010568;...
               +0.90617984593866,  +0.53846931010568;...
               -0.90617984593866,  +0.90617984593866;...
               -0.53846931010568,  +0.90617984593866;...
               +0               ,  +0.90617984593866;...
               +0.53846931010568,  +0.90617984593866;...
               +0.90617984593866,  +0.90617984593866];
         Weights=[...
               0.05613434886243;...
               0.11340000000000;...
               0.13478507238752;...
               0.11340000000000;...
               0.05613434886243;...
               0.11340000000000;...
               0.22908540422399;...
               0.27228653255075;...
               0.22908540422399;...
               0.11340000000000;...
               0.13478507238752;...
               0.27228653255075;...
               0.32363456790123;...
               0.27228653255075;...
               0.13478507238752;...
               0.11340000000000;...
               0.22908540422399;...
               0.27228653255075;...
               0.22908540422399;...
               0.11340000000000;...
               0.05613434886243;...
               0.11340000000000;...
               0.13478507238752;...
               0.11340000000000;...
               0.05613434886243];
         
      otherwise
         disp (['Not implemented for quadrilateral area '...
               'NumPoints = ',num2str(NumPoints)]);
         error('Stopping');
      end
   %------------------------------------------------------------------   
   otherwise 
      disp ([' A ',num2str(NumSideElem),...
            ' side element was specified which will not be used']);
      error('Stopping');
      
   end
end

      