%**************************************************************************
% This file is part of the MLS-C Library; you can redistribute it and/or 
% modify it under the terms of the GNU Lesser General Public License as 
% published by the Free Software Foundation; either version 3 of the 
% License,or any later version.
% This program is distributed in the hope that it will be useful but 
% WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
% Public License for more details.
% You should have received a copy of the GNU Lesser General Public License
% along with GNU; see the file COPYING.  If not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
% or see <http://www.gnu.org/licenses/>.
%**************************************************************************
% Created by: 
%    E Nadal, M Ouallal, JJ Rodenas, F Chinesta and FJ Fuenmayor
%  
% Release 1.0  
% Date: 18/07/2014
%**************************************************************************

function [BoundPts,AngleBoundPts,tBoundPts,BoundNum,NodeType,NBP]=...
    ClosestPointsOnBoundrs(x,y,ds,GeoPoints,Boundaries)

% This function finds the closest points to (x,y) on each of the boundaries
% Points are within the support of (x,y), i.e. at a distance dist2xy, 
% dist2xy<=ds

BoundPts      = [];
AngleBoundPts = [];
BoundNum      = [];
NodeType      = [];
tBoundPts     = [];
dist2xy       = [];

xy = [x y];

for i=1:size(Boundaries,1)
    v1=GeoPoints(Boundaries(i,1),:);
    v2=GeoPoints(Boundaries(i,2),:);
    
    %find nearest point only if curve is of
    %type==40 (Newman boundary) or type=41 (symmetry boundary)
    if Boundaries(i,5)==40 || Boundaries(i,5)==41
        
        if Boundaries(i,3)==0 %Curve is of type LINE SEGMENT
            
            %Translate segment limits and point
            a  = xy-v1;
            b  = [0,0]; %v1-v1;
            c  = v2-v1;
            % Rotate coordinates of point and 2nd vertex of segment
            [theta,~]=cart2pol(c(1),c(2));
            M   =[cos(-theta),-sin(-theta);sin(-theta),cos(-theta)];
            xyPoint =M*a';
            xyV2    =M*c';
            
            %Check point possition with repect to segment
            if xyPoint(1)<=0 %point is on the left of the segment
                xyNearest= v1;
                t=0;
            elseif xyPoint(1)>=xyV2(1) %point is on the right of segment
                xyNearest= v2;
                t=1;
            else %point is above or below the segment
                xyNearest=[xyPoint(1) 0];
                t=(xyNearest(1)-0)/(xyV2(1)-0);
                M   =[cos(theta),-sin(theta);sin(theta),cos(theta)];
                xyNearest= (M*xyNearest'+v1')';
            end
            
            
            AngleBoundPts = [AngleBoundPts; theta-pi()/2];
          
            
        elseif Boundaries(i,3)>0 %Curve is of type ARC OF CIRCUMFERENCE
            %Translate segment limits and point
            vc = GeoPoints(Boundaries(i,3),:);
            %Translate coordinates system to center of circumf.
            PointCart  = xy-vc;
            CenterCart = [0,0]; %vc-vc;
            Point1Cart = v1-vc;
            Point2Cart = v2-vc;
            %Evaluate angle of each point
            PointAng      =cart2pol(PointCart(1), PointCart(2) );
            [Point1Ang,R] =cart2pol(Point1Cart(1),Point1Cart(2));
            Point2Ang     =cart2pol(Point2Cart(1),Point2Cart(2));
            
            ClockWise=Boundaries(i,4);
            if ClockWise>=0; ClockWise=1;
            else ClockWise=0;
            end
            
            %The general case (does not require any adjustment) is:
            %      ((Point1Ang<Point2Ang) && ~ClockWise) ||...
            %      ((Point2Ang<Point1Ang) &&  ClockWise)
            
            if ((Point1Ang<Point2Ang) && ClockWise)
                Point2Ang=Point2Ang-2*pi();
            elseif ((Point2Ang<Point1Ang) && ~ClockWise)
                Point2Ang=Point2Ang+2*pi();
            end
            
            minAng=min(Point1Ang,Point2Ang);
            maxAng=max(Point1Ang,Point2Ang);
            Points2Check=[PointAng-2*pi,PointAng,PointAng+2*pi];
            
            %Check points possition with repect to segment
            NearestAng=[]; AngDif=[];
            
            for j=1:3
                P=Points2Check(j);
                
                if P <= minAng %point on the left of segment
                    NearestAng(j) = minAng;
                    AngDif(j)     = minAng-P;
                elseif P >= maxAng %point on the right of segment
                    NearestAng(j) = maxAng;
                    AngDif(j)     = P-maxAng;
                else %point is above or below the segment
                    NearestAng(j) = P;
                    AngDif(j)     = 0;
                end
            end
            [~,IndexOfMinAngleDif] =min(AngDif);
            NearestAng=NearestAng(IndexOfMinAngleDif);
            t=(NearestAng-minAng)/(maxAng-minAng);
            
            [xyNearest(1),xyNearest(2)]= pol2cart(NearestAng,R);
            xyNearest=xyNearest+vc;
            
            if ClockWise
                NearestAng=NearestAng-pi;
            end
            AngleBoundPts = ...
                [AngleBoundPts; NearestAng];
                
        end  
    end
    
    dist2xy            = [dist2xy,norm([x y] - xyNearest)];
    BoundPts  = [BoundPts;  xyNearest      ];
    tBoundPts = [tBoundPts; t              ];
    BoundNum  = [BoundNum;  i              ];
    NodeType  = [NodeType;  Boundaries(i,5)];
    
end

%% Select only the points within the support ds
index         = find(dist2xy<=ds);
BoundPts      = BoundPts(index,:);
AngleBoundPts = AngleBoundPts(index);
tBoundPts     = tBoundPts(index);
BoundNum      = BoundNum(index);
NodeType      = NodeType(index);
NBP           = length(index);


%% Activate the following code if you want to plot the nearest points
%to a given point in each of the curves used to define the boundary
%{
PlotGeo
% plot(BoundPts(:,1),BoundPts(:,2),'r*');
SizeX=max(GeoPoints(:,1))-min(GeoPoints(:,1));
ArrowSize=SizeX*.1;
for i=1:size(BoundPts,1)
    text(BoundPts(i,1),BoundPts(i,2),num2str([BoundNum(i) tBoundPts(i)]));
    plot(x,y,'+r')
    plot(...
        [BoundPts(i,1),BoundPts(i,1)+ArrowSize*cos(AngleBoundPts(i))],...
        [BoundPts(i,2),BoundPts(i,2)+ArrowSize*sin(AngleBoundPts(i))],...
        'r-');
end
%}
