%**************************************************************************
% This file is part of the MLS-C Library; you can redistribute it and/or 
% modify it under the terms of the GNU Lesser General Public License as 
% published by the Free Software Foundation; either version 3 of the 
% License,or any later version.
% This program is distributed in the hope that it will be useful but 
% WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
% Public License for more details.
% You should have received a copy of the GNU Lesser General Public License
% along with GNU; see the file COPYING.  If not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
% or see <http://www.gnu.org/licenses/>.
%**************************************************************************
% Created by: 
%    E Nadal, M Ouallal, JJ Rodenas, F Chinesta and FJ Fuenmayor
%  
% Release 1.0  
% Date: 18/07/2014
%**************************************************************************
%%
function [FieldStGP]= MLS( FieldFEGP,XYZGP,XYZout )
%
%% THE MOVING LEAST SQUARES (MLS) RECOVERY TECHNIQUE
% This Matlab(c) subroutine uses a Moving Least Squares approach to 
% evaluate a recovered stress field from a 2D stress field evaluated using 
% the finite element method.
% The mathematical basis used to create this subroutine can be found in
% J.J. Rdenas et al 2012.
%
% A basic description of this subroutine can be found in the file 
% MLStechnique.pdf.

%% INPUT AND OUTPUT
%-------------------------------------------------------------------------- 
% INPUT VARIABLES
%   FieldFEGP   = FE stresses at sampling (integration) points
%   XYZGP       = Coordinates of the sampling (integration) points
%   XYZout      = Coordinates of the points where the recovered stress 
%                 field has to be evaluated
%   ProblemDirF = Working directory
% OUTPUT VARIABLES
%   FieldStGP = Reecovered stresses at XYZout
%
% MAIN INTERNAL VARIABLES
% 
%--------------------------------------------------------------------------
% Created by: 
%    E Nadal, M Ouallal, JJ Rodenas and FJ Fuenmayor
%
% Date: 11/03/2013
%--------------------------------------------------------------------------

%% RESHAPE INPUT DATA
% Input data is reshaped if needed 
if size(FieldFEGP,2)> size(FieldFEGP,1)
    FieldFEGP = FieldFEGP';
end
if size(XYZGP,2)>size(XYZGP,1)
    XYZGP     = XYZGP';
end
if size(XYZout,2)>size(XYZout,1)
    XYZout    = XYZout';
end

%% DEFENITION OF SUBROUTINE PARAMETERS

% Internal codes
% --------------
load('GeoPoints.txt');
load('Boundaries.txt');

BoundImpTract       =40;                 % Boundary node with known
                                         % Normal and Tang stresses
Symmetry            =41;                 % Symmetry BC
IntEquil            = 1;                 % Internal equilibrium
FIE                 = 1;                 % Full internal equilibrium
ContourEquilWprime  = 1;                 % Contour equilibrium
TolBundary          = 1e-10;             % Tolerance for bound. conditions
                                         % Used to check if a point is over
                                         % a node on the boundary

% General configuration
% ---------------------
Noc                 = size(FieldFEGP,2); % Number of field components
DrawSupport         = 0;                 % Flag 0 plot / 1 do not plot
                                         % support of each point
SubSizes            = [3,6,10,15,21,28]; % Size of equation system for each
                                         % polynomial degree of recovered
                                         % stress field
MaxFieldDegree      = 3;                 % Degree of the recovered field
NumGP               = size(XYZGP,1);     % Total number of points in XYZGP
Numout              = size(XYZout,1);    % Total number of points in XYZout
NumInflPoints       = 151;               % Number of influence points in 
                                         % the support

disp(['The number of influence points considered in the support is: '...
    num2str(NumInflPoints) '.'])

MinDegree = 100;                % Keep polynomial expansion degree used 
                                % for each GP.
CondNum   = zeros(NumGP,1);     % Initilize the number of GP verctor.

%% ERROR ESTIMATION EQUATIONS
% The ZZ error estimator will be evaluated as follows:
%
% $$ ||\mathbf{e}||^2 = \int_{\Omega} ({\sigma}^*-{\sigma}^h)^T 
% \mathbf{D}^{-1} ({\sigma}^*-{\sigma}^h)d\Omega $$
%%
% The recovered stress field components are obtained from a polynomial
% expansion, using the following expression: 
%
% $$ \sigma_{i} ^{*} (\mathbf{x}) = 
% \mathbf{p}(\mathbf{x})\mathbf{a}_i(\mathbf{x})\quad i=xx,yy,xy $$
%
% For each stress components in the 2D case we would have:
%
% $$ \sigma^{*} (\mathbf{x}) = \left [\begin {array}{ccc}\sigma_{xx}^{*}(x)
% \\\noalign{\medskip}\sigma_{yy}^{*}(x)
% \\\noalign{\medskip}\sigma_{xy}^{*}(x)\end{array}\right] = 
% \mathbf{P}(\mathbf{x})\mathbf{A}(\mathbf{x}) =
% \left [\begin {array}{ccc} p(x) & 0 & 0\\\noalign{\medskip}0 & p(x) & 0
% \\\noalign{\medskip}0 & 0 & p(x) \end{array}\right] 
% \left [\begin {array}{ccc} a_{xx}(x)\\\noalign{\medskip}a_{yy}(x)
% \\\noalign{\medskip}a_{xy}(x) \end{array}\right] $$


%% MAXIMUM INFLUENCE RADIUS ASSUMING UNIFORM SAMPLING POINTS DISTRIBUTION
% Now, the MLS technique considers a point $$ \chi $$ in a support 
% denominated $$ \Omega_x $$ corresponding to a point $$ \mathbf{x} $$ 
% defined by a radius $$ R_{\Omega_{x}} $$ .
% So the MLS approximation is given by:
%
% $$ \sigma_{i} ^{*} (\mathbf{x},\mathbf{\chi}) =
% \mathbf{p}(\mathbf{\chi})\mathbf{a}_i(\mathbf{x}) \quad
% \forall\chi\in\Omega_x,\quad i=xx,yy,xy $$
%%

XMin         = min(XYZGP(:,1)); 
XMax         = max(XYZGP(:,1));
YMin         = min(XYZGP(:,2)); 
YMax         = max(XYZGP(:,2));
RadProb      = sqrt((XMax-XMin)^2 + (YMax-YMin)^2)/2;
Area         = pi() * RadProb^2;
PointDensity = NumGP / Area;
MaxRadInf    = sqrt(4*NumInflPoints/(PointDensity * pi()));
disp(['The influence radius is: ' num2str(MaxRadInf) ...
    ' and the problem radius is: ' num2str(RadProb)]);


%% MSL LOOP AND DEFINITION OF EQUATIONS TO OBTAIN THE RECOVERED FIELD.
% In order to obtain the recovered stress field $$ \mathbf\sigma_{i} ^{*} (\mathbf{x}) =
% {p}(\mathbf{x}){a}_i(\mathbf{x})\quad i=xx,yy,xy $$
% we have to solve the linear system of equations $$ \mathbf{M}(\mathbf{x})\mathbf{A}(\mathbf{x})=\mathbf{G}(\mathbf{x}) $$ so
% we can get the vector of coefficient $$ \mathbf{A}(\mathbf{x}) $$. 
%

% Initilize output matrix
% -----------------------

FieldStGP = zeros(Numout,Noc);
Auxds     = zeros(Numout,1);

for iGP = 1:Numout
    %% Inilialize variables in case Lagrange for boundary
    %  --------------------------------------------------
    Madd = [];                % Main part of the system matrix
    Cadd = [];                % Constraint equations 
    
    %% Evaluation of points within the support of the assembly point
    
    % Find influence GP.
    % -------------------
    XYPt = XYZout(iGP,:);               % Coordinates of the assembly GP
    
    if NumGP <= NumInflPoints 
        InflPoints     = 1:NumGP;
    else
        dummyXGP       = (XYZGP(:,1)-XYPt(1));
        dummyYGP       = (XYZGP(:,2)-XYPt(2));
        dummyRadXYZGP  = sqrt(dummyXGP.^2 + dummyYGP.^2);
        SuitablePoints = find(dummyRadXYZGP <= MaxRadInf);
        [~,I]          = sort(dummyRadXYZGP(SuitablePoints),'ascend');
        
        if length(I) > NumInflPoints   % Find number of influence points
            InflPoints = SuitablePoints(I(1:NumInflPoints));
        else
            InflPoints = SuitablePoints(I);
        end
    end
    
    
    NSP        = length(InflPoints);               % Number of sampling points
    ds         = max(sqrt((XYZGP(InflPoints,1)-XYPt(1)).^2 +...
                (XYZGP(InflPoints,2)-XYPt(2)).^2));% Influence radius
    Auxds(iGP) = ds;
    
    %%
    % There are $$ n $$ sampling points of coordinates $$ \chi $$ within
    % the support using the FE analysis where the stresses are already
    % available.
    % The normalized distance function is given to calculate the the
    % weighting function: 
    %
    % $$ s = \frac{\left\|\mathbf{x}-\mathbf{\chi}  \right\| } {R_{\Omega_{\mathbf{x}}} } $$
    
    
    
    % Local coordinates of the influence points
    % -----------------------------------------
    
    X = (XYZGP(InflPoints,1) - XYPt(1))/ds; 
    Y = (XYZGP(InflPoints,2) - XYPt(2))/ds;
    S = sqrt(X.^2 + Y.^2);
    
    
    % Check if the AssGP support in in contact with any boundary
    % ----------------------------------------------------------
                           
    [BoundPts,AngleBoundPts,tBoundPts,BoundNum,NodeType,NBP]...
        = ClosestPointsOnBoundrs(XYPt(1), XYPt(2),ds,GeoPoints,Boundaries);

    
  
    % Local coordinates of the influence points on the boundaries.
    % ------------------------------------------------------------
    XBP = ( BoundPts(:,1)-XYPt(1) )/ds; 
    YBP = ( BoundPts(:,2)-XYPt(2) )/ds;
    
    
    % Distance to the boundary points.
    % --------------------------------
    SBP = ( XBP.^2 + YBP.^2 ).^0.5; 
    
    
    % Draw support and influence points
    % ---------------------------------
    if DrawSupport ==1
        figure;
        hold on;
        PlotGeo
        hold on;
        rectangle('Position',[XYPt(1)-ds,XYPt(2)-ds,2*ds,2*ds],...
                  'Curvature',[1,1]);
        plot(XYPt(1),XYPt(2),'bo');
        plot(XYZGP(InflPoints,1),XYZGP(InflPoints,2),'r.');
        if NBP~=0
            plot(BoundPts(:,1),BoundPts(:,2),'kO');
        end
        hold off
        axis equal
        pause;
    end
    
    %%
    % The weighting function has been taken as the fourth-order spline,
    % commonly used in the MLS related literature: 
    % 
    % $$ W (\mathbf{x}-\mathbf{\chi} ) = 
    % 1-6s^{2} +8s^{3} -3s^{4} $$
    
    % Evaluate the Weighting Function
    % -------------------------------
    S2     =  S.*S;
    W      = (1 - 6*S2 + 8*S2.*S - 3*S2.*S2);
    W(W<0) =  0; % needed due to round-off errors
    
    
    % Create PatchMatrices and impose constraint equations.
    % -----------------------------------------------------
    if IntEquil
        dummy    = (-12+24*S-12*S.^2);
        Diameter = ds;
        dWdx     = dummy.*X/Diameter^2;
        dWdy     = dummy.*Y/Diameter^2;
    end
    
    WpFactor = 1;
    
    %%
    % The boudary equilibrium must be satisfied at each point along the
    % contour and in order to obtain a continous recovered stress field a  
    % _nearest point_ approach. 
    % being _NBP_ the number of points on the boundary where the
    % constraints are imposed ( see the pdf file)
    % 
    % In that case, the weighting function is defined as:
    %
    % $$ W' (\mathbf{x}-\mathbf{\chi}_{j} ) = 
    % \frac{W (\mathbf{x}-\mathbf{\chi}_{j} )}{s}=
    % \frac{1}{s} -6s +8s^{2} -3s^{3} $$ 
    
    % Evaluate the Weighting Function in Boundaries
    % ---------------------------------------------
    if ContourEquilWprime && NBP > 0 
        WBP = abs(1./SBP - 6*SBP + 8*SBP.^2 - 3*SBP.^3)*WpFactor; 
        if IntEquil
            dummy  = (-1./SBP.^3 - 6./SBP + 16 - 9*SBP)*WpFactor;
            dWBPdx = dummy.*XBP/Diameter^2;
            dWBPdy = dummy.*YBP/Diameter^2;
        end
    end
    
    % Evaluate Degree of interpolation polynomial in patch. 
    % -----------------------------------------------------
    
    % Remember that SubSizes = [3,6,10,15,21,28];
    Degree = MaxFieldDegree;
    while NSP<SubSizes(Degree)
        Degree = Degree-1;
    end
    
    if Degree < 1
        disp...
      ('Not enough sampling points. Please reconsider the discretization');
    end
    
    SubMSize       = SubSizes(Degree);
    MinDegree(iGP) = Degree;
    
    %% Prepare data for *M* and *G* matrices for internal and boundary equ.
    %
    %
    % Before Getting through the imposition of the boundary and internal
    % equilibrium equtions, we prepare the equation system for *M* and *G*
   
    
    % Prepare data for M and G matrices
    % ---------------------------------
    [pTerms]  = polininterp2d(X,Y,Degree);
    RowOfOnes = ones(1,SubMSize);
    TotalSize = SubMSize*Noc;
    M         = zeros(TotalSize,TotalSize);
    
        % Internal equilibrium case.
        % **************************
    
    if IntEquil; 
        dMdx  = M; 
        dMdy  = M; 
    end
    
            % Create original system of equations
            % ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        
    W1        =  W * RowOfOnes;
    pTermsTr  =  pTerms';
    pTermsTrW =  pTermsTr.*W1';
    Wptp      =  pTermsTr*(pTerms.*W1);
    G         = [pTermsTrW*FieldFEGP(InflPoints,1);...% System of load equation
                 pTermsTrW*FieldFEGP(InflPoints,2);...
                 pTermsTrW*FieldFEGP(InflPoints,3)];
    
            % Loop over field components
            % ~~~~~~~~~~~~~~~~~~~~~~~~~~
        
    for iNoc=1:Noc
        Index  = SubMSize*(iNoc-1)+1;
        M(Index:Index+SubMSize-1,Index:Index+SubMSize-1)...
               = Wptp;
    end
    
    if IntEquil 
        dWdx1        =  dWdx * RowOfOnes; 
        dWdy1        =  dWdy * RowOfOnes;
        pTermsTrdWdx =  pTermsTr.*dWdx1';
        pTermsTrdWdy =  pTermsTr.*dWdy1';
        dWdxptp      =  pTermsTr*(pTerms.*dWdx1);
        dWdyptp      =  pTermsTr*(pTerms.*dWdy1);
        dGdx         = [pTermsTrdWdx*FieldFEGP(InflPoints,1);...
                        pTermsTrdWdx*FieldFEGP(InflPoints,2);...
                        pTermsTrdWdx*FieldFEGP(InflPoints,3)];
        dGdy         = [pTermsTrdWdy*FieldFEGP(InflPoints,1);...
                        pTermsTrdWdy*FieldFEGP(InflPoints,2);...
                        pTermsTrdWdy*FieldFEGP(InflPoints,3)];
        
        for iNoc=1:Noc 
            Index = SubMSize*(iNoc-1)+1;
            dMdx(Index:Index+SubMSize-1,Index:Index+SubMSize-1)...
                  = dWdxptp;
            dMdy(Index:Index+SubMSize-1,Index:Index+SubMSize-1)...
                  = dWdyptp;
        end
    end
    
    
    %% Boundary and internal equilibrium enforcement.
    %
    %
    % Hereinafter the part of the code showing the process to add terms
    % to *M* and *G* in both; boundary and internal equilibrium.
    % We have along the support $$ \Omega_\mathbf{x}\chi $$:
    %
    % $$ \mathbf{M}(\mathbf{x})= \int_{\Omega_{\mathbf{x}}} 
    % W(\mathbf{x}-\mathbf{\chi})\mathbf{P}^{T}(\mathbf{\chi})\mathbf{P}(\mathbf{\chi})d{\chi} $$
    % 
    % $$ \mathbf{G}(\mathbf{x})=\int_{\Omega_{\mathbf{x}}} W(\mathbf{x}-\mathbf{\chi})
    % \mathbf{P}^{T}(\mathbf{\chi})\mathbf{\sigma}^{h}(\mathbf{\chi})d{\chi} $$
    % 
    % Assuming _n_ sampling points of coordinates $$ \mathbf\chi_l $$ we would
    % numerically have:
    %
    % $$ \mathbf{M} = \sum _{l=1}^{n} W(\mathbf{x}-\mathbf{\chi}_{l})
    % \mathbf{P}^{T}(\mathbf{\chi}_{l})\mathbf{P}(\mathbf{\chi}_{l})|{J}(\mathbf{\chi}_{l})|H_l $$
    %
    % $$ \mathbf{G} = \sum _{l=1}^{n} W(\mathbf{x}-\mathbf{\chi}_{l})
    % \mathbf{P}^{T}(\mathbf{\chi}_{l})\mathbf{\sigma} ^{h}(\mathbf{\chi}_{l})|{J}(\mathbf{\chi}_{l})|H_l $$
    %
  
    % Adding constraints
    % ------------------
    
    
        % Add terms related to boundary conditions.
        % *****************************************
    
    if ContourEquilWprime && NBP > 0 
        for i=1:NBP
            %%
            % Along the boundaries, the stress vector $$ \sigma^*(x,\chi) $$
            % in the local coordinate system $$ \widetilde{x}\widetilde{y} $$
            % is expressed :
            %
            % $$ \widetilde\sigma^*(\mathbf{x},\mathbf{\chi})=r(\alpha)\mathbf{\sigma}^*(\mathbf{x},\mathbf{\chi}) $$
            %
            % where *r* is the stress rotation matrix:
            % 
            % $$ r 
            % = \left [\begin {array}{ccc}{r}_{\widetilde{x}\widetilde{x}}
            % \\\noalign{\medskip}{r}_{\widetilde{y}\widetilde{y}}
            % \\\noalign{\medskip}{r}_{\widetilde{x}\widetilde{y}}
            % \end{array}\right] = 
            % \left [\begin {array}{ccc} \cos^2 \alpha & \sin^2 \alpha  
            % & \sin(2\alpha)\\\noalign{\medskip}sin^2\alpha  
            % & \cos^2 \alpha  & -\sin(2\alpha)
            % \\\noalign{\medskip}-\sin(2\alpha)/2 
            % & \sin(2\alpha)/2 & \cos(2\alpha) \end{array}\right]  $$
            %
            %
            
            Ang          = AngleBoundPts(i);
            pBP          = polininterp2d...           % Evaluate polynomial
                          (XBP(i),YBP(i),Degree,1);   % on boundaries.
            LocaDistance = sqrt(XBP(i)^2 + YBP(i)^2); % Evaluate distance
                                                      % from the evaluation
                                                      % point to boundary
                                                      % point.
            if LocaDistance > TolBundary
                Lagrange = 0; % If far, normal way
            else 
                Lagrange = 1; % If close, Lagrange multipliers
            end
            switch NodeType(i)
                case BoundImpTract 
                    
                    % Evaluate stress at boundary point(Normal & Tangential)
                    % ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

                                              
                    [~,SIG]=press(BoundNum(i),tBoundPts(i),...
                        BoundPts(i,1),BoundPts(i,2),AngleBoundPts(i));
                                              
                    
                    % Prepare the Rotation matrices
                    % ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                    [~,RStress] = RotationMatrices(Ang,2);
                    
                    PTotalNode  = zeros(Noc,TotalSize);
                    for iNoc=1:Noc
                        Index = SubMSize*(iNoc-1)+1;
                        PTotalNode(iNoc,Index:Index+SubMSize-1)...
                              = pBP;
                    end
                    
                    
                    % Sxx and Sxy stress at contour
                    % ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                    RPxx         = RStress(1,:)*PTotalNode;
                    RPxy         = RStress(3,:)*PTotalNode;
                    PtRtRP       = RPxx'*RPxx   + RPxy'*RPxy  ;
                    PtRtSigt     = RPxx'*SIG(1) + RPxy'*SIG(2);
                    
                case Symmetry 
                    % Evaluate stresses at symmetric side
                    % ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                    [~,RStress]  = RotationMatrices(Ang,2);
                    PTotalNode   = zeros(Noc,TotalSize);
                    for iNoc=1:Noc 
                        Index    = SubMSize*(iNoc-1)+1;
                        PTotalNode(iNoc,Index:Index+SubMSize-1)...
                                 = pBP;
                    end
                    %Sxy stress at contour = 0
                    RPxy         = RStress*PTotalNode;
                    RPxy         = RPxy(3,:);
                    PtRtRP       = RPxy'*RPxy;
                   
                otherwise  %Change this for other types of nodes
                    PtRtRP       = sparse(size(M,1),size(M,1));
                    PtRtSigt     = sparse(size(G,1),1);
                    
            end
        %%
        % For _NBP_ points in the boundary, the *M* and *G* are in this
        % case:
        %
        % $$ \mathbf{M} = \sum _{l=1}^{n} W(\mathbf{x}-\mathbf{\chi}_{l})\mathbf{P}^{T}(\mathbf{\chi}_{l})
        % \mathbf{P}(\mathbf{\chi}_{l})|{J}(\mathbf{\chi}_{l})|H_l+
        % \sum _{j=1}^{nbc} W'(\mathbf{x}-\mathbf{\chi}_{j}) 
        % \mathbf{P}^{T} (\mathbf{\chi}_{j}) {r}_{\widetilde{i}}^T {r}_{\widetilde{i}} 
        % \mathbf{P} ({\chi}_{j}) $$
        %
        % $$ \mathbf{G} = \sum _{l=1}^{n} W(\mathbf{x}-\mathbf{\chi}_{l})\mathbf{P}^{T}(\mathbf{\chi}_{l})
        % \mathbf{\sigma} ^{h}(\mathbf{\chi}_{l})|{J}(\mathbf{\chi}_{l})|H_l+
        % \sum _{j=1}^{nbc} W' (\mathbf{x}-\mathbf{\chi}_{j})
        % \mathbf{P}^{T}(\mathbf{\chi}_{j}){r}_{\widetilde{i}}^T
        % {\sigma'}_{\widetilde{i}} ^{ex} (\mathbf{\chi}_{j}) $$
        
               
        % Adding equations to M an G matrices
        % ************************************
             % Add equations to M an G
             % ~~~~~~~~~~~~~~~~~~~~~~~
             
            if ~Lagrange
                M = M + WBP(i)*PtRtRP;
                if NodeType(i)~=Symmetry % if NodeType==Symmetry => Sig=Tau=0
                    G = G + WBP(i)*PtRtSigt;
                end
                if IntEquil
                    dMdx = dMdx + dWBPdx(i)*PtRtRP;
                    dMdy = dMdy + dWBPdy(i)*PtRtRP;
                    if NodeType~=Symmetry % if NodeType==Symmetry => Sig=Tau=0
                        dGdx = dGdx + dWBPdx(i)*PtRtSigt;
                        dGdy = dGdy + dWBPdy(i)*PtRtSigt;
                    end
                end
            else
                if  NodeType(i)~=Symmetry
                    Madd = [Madd ; RPxx ; RPxy];
                    Cadd = [Cadd ; SIG(1) ; SIG(2)];
                else
                    Madd = [Madd ; RPxy];
                    Cadd = [Cadd ; 0];
                end
                break
            end
        end
    end
        %%
        % The internal equilibrium equation is satisfied using the Lagrange
        % multipliers technique.
        % (see the pdf file) 
        %
        % The *M* and *G* could be obtained from its derivatives:
        % 
        % $$ \frac{\partial\mathbf{M}}{\partial\mathbf{x}} =
        % \sum _{l=1}^{n} \frac{\partial W (\mathbf{x}-\mathbf{\chi}_{l})}{\partial\mathbf{x}}
        % \mathbf{P}^{T}  (\mathbf{\chi}_{l})\mathbf{P} (\mathbf{\chi}_{l})  |{J}(\mathbf{\chi}_{l})|H_l+
        % \sum _{j=1}^{nbc}\frac{\partial W' (\mathbf{x}-\mathbf{\chi}_{j})}{\partial\mathbf x}
        % \mathbf{P}^{T} (\mathbf{\chi}_{j}){r}_{\widetilde{i}}^T{r}_{\widetilde{i}}
        % \mathbf{P}(\mathbf{\chi}_{j}) $$
        % 
        % 
        % $$ \frac{\partial\mathbf{G}}{\partial\mathbf x} =
        % \sum _{l=1}^{n} \frac{\partial W (\mathbf{x}-\mathbf{\chi}_{l})}{\partial\mathbf{x}}
        % \mathbf{P}^{T} (\mathbf{\chi}_{l})\mathbf{\sigma^h} (\mathbf{\chi}_{l}) |{J}(\mathbf{\chi}_{l})|H_l+
        % \sum _{j=1}^{nbc}\frac{\partial W' (\mathbf{x}-\mathbf{\chi}_{j})}{\partial\mathbf{x}}
        % \mathbf{P}^{T}(\mathbf{\chi}_{j}){r}_{\widetilde{i}}^T
        % \mathbf\sigma_{\widetilde{i}}^{ex}(\mathbf{\chi}_{j}) $$
        
    
        % Add internal equilibrium constraint equations which
        % satisfy contour equilibrium.
        % ***************************************************
    if IntEquil
       
        % MAIN EQUATIONS (related to x-direction derivatives)
     
            % pTerms and its partial derivatives
            % ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        [p0,dp0dx,dp0dy]=polininterp2d(0,0,Degree,1);
        
        Minv  = inv(M);
        P0    = zeros(Noc,TotalSize);
        dP0dx = P0;
        dP0dy = P0;
        for iNoc=1:Noc
            Index                              = SubMSize*(iNoc-1)+1;
            P0   (iNoc,Index:Index+SubMSize-1) = p0;
            dP0dx(iNoc,Index:Index+SubMSize-1) = dp0dx;
            dP0dy(iNoc,Index:Index+SubMSize-1) = dp0dy;
        end
        T1x      =  dP0dx - FIE*P0*Minv*dMdx;  
        T2x      =  FIE*P0*Minv*dGdx;
        T1y      =  dP0dy - FIE*P0*Minv*dMdy;      
        T2y      =  FIE*P0*Minv*dGdy;
        MConstr  = [T1x(1,:)+T1y(3,:);...
                    T1y(2,:)+T1x(3,:)];
        
            % Evaluate body forces
            % ~~~~~~~~~~~~~~~~~~~~
        [b]      =  VolLoads(XYPt,[],[]);
         GConstr = [-T2x(1,:)-T2y(3,:)-b(1);...
                    -T2y(2,:)-T2x(3,:)-b(2)];
         M       = [ M       , MConstr';...
                     MConstr , zeros(size(MConstr,1),size(MConstr,1))];
         G       = [ G; GConstr ];
    end
    %% Obtaining the vector of coeficient *A*.
    %
    %
    % The use of Lagrange Multipliers leads to the system of equation: 
    %
    % $$  \left [\begin {array}{ccc}\mathbf{M} &
    % \mathbf{C}^T\\\noalign{\medskip}\mathbf{C} & \mathbf{0}
    % \end{array}\right] 
    % \left [\begin {array}{ccc} \mathbf{A}\\\noalign{\medskip}\mathbf{\lambda} 
    % \end{array}\right]   =
    % \left [\begin {array}{ccc} \mathbf{G}\\\noalign{\medskip}\mathbf{D}
    % \end{array}\right]  $$
    %
    % The resolution of that system, is processed by blocks.
    % (more explication in the pdf file).
    
    
    % Solving The System of Equation
    % ------------------------------
    CondNum(iGP) = condest(M);
    
        % If boundary conditions with Lagrange Multipliers
        % *************************************************
    if (ContourEquilWprime && NBP > 0)  && Lagrange
        if size(Madd,2) < size(M,2)
            Madd(size(Madd,1),size(M,2)) = 0;
        end
        M = [M Madd';Madd sparse(size(Madd,1),size(Madd,1))];
        G = [G; Cadd];
    end
    
    Coefi            = M\G;
    FieldStGP(iGP,:) = Coefi(1:SubMSize:((Noc-1)*SubMSize+1))';
end
disp(['MinDegree = ' num2str(min(MinDegree))]);

disp('Smoothing technique finished. All data saved');
