%**************************************************************************
% This file is part of the MLS-C Library; you can redistribute it and/or 
% modify it under the terms of the GNU Lesser General Public License as 
% published by the Free Software Foundation; either version 3 of the 
% License,or any later version.
% This program is distributed in the hope that it will be useful but 
% WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
% Public License for more details.
% You should have received a copy of the GNU Lesser General Public License
% along with GNU; see the file COPYING.  If not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
% or see <http://www.gnu.org/licenses/>.
%**************************************************************************
% Created by: 
%    E Nadal, M Ouallal, JJ Rodenas, F Chinesta and FJ Fuenmayor
%  
% Release 1.0  
% Date: 18/07/2014
%**************************************************************************
%%
% This Matlab(c) subroutine calls the Moving Least Squares function to 
% evaluate a recovered stress field from a 2D stress field evaluated using 
% the finite element method for quadratic elements.

% Remove all blanck lines and headers from the output files (Element 
% solution, Topology, Node Coordinates) generated by Ansys to be used as 
% input argument when calling the MLS subroutine.

AnsysRelease = 14; %Valid releases are 12 and 14

%% Read NLIST.txt   Nodal information
f = fopen('NLIST.txt', 'rt');
if AnsysRelease ==14
    c = textscan(f,'%s','Delimiter','%s','headerlines',4,...
        'CollectOutput',true); %Read file
    lines = c{1};              %Store nodal coords
elseif AnsysRelease == 12
    c = textscan(f,'%s','Delimiter','%s','headerlines',3,...
        'CollectOutput',true); %Read file
    lines = c{1};              %Store nodal coords
    lines(1:21:end) = [];      %Remove blank lines
else
    err ('Ansys release not supported');
end 
fclose(f);

coordinatesXY = zeros(numel(lines),4);

for i = 1:numel(lines)
    coordinatesXY(i,:) = coordinatesXY(i,:) + str2num(lines{i});

end

coordinatesXY(:,[1,4]) = [];
Xcoord = coordinatesXY(:,1);
Ycoord = coordinatesXY(:,2);

%% Read ELIST.txt   Element information

f = fopen('ELIST.txt','rt');
if AnsysRelease ==14
    c = textscan(f,'%s','Delimiter','%s','headerlines',4,...
        'CollectOutput',true);
    lines = c{1};
elseif AnsysRelease == 12
    c = textscan(f,'%s','Delimiter','%s','headerlines',2,...
        'CollectOutput',true);
    lines = c{1};              
    lines(1:21:end) = [];      
else
    err ('Ansys release not supported');
end 
fclose(f);
 
topology = zeros(numel(lines),14);

for i = 1:numel(lines)
    topology(i,:) = topology(i,:) + str2num(lines{i});

end

topology(:,1:6) = [];

%

f = fopen('PRESOL.txt','rt');
c = textscan(f,'%s','Delimiter','%s','headerlines',2,...
    'CollectOutput',true);

lines= c{1};
lines(1:6:end)=[];
lines(1:5:end)=[];
GaussStresses = zeros(numel(lines),7);
NumGP         = 4;
NumElem       = size(topology,1);

for i = 1:numel(lines)
    GaussStresses(i,:) = GaussStresses(i,:) + str2num(lines{i});

end

GaussStresses(:,[4,6,7]) = [];

Temporary_Quad = mat2cell(GaussStresses,NumGP*ones(NumElem,1),NumGP);

for i = 1:NumElem
Temp = Temporary_Quad{i,1};
    for k = 2:NumGP
        if Temp(k,1)==Temp(k-1,1)
            Temp(k-1,:)=[];
        end
    end
Temporary_Quad{i,1}=Temp;
end

GaussStresses    = cell2mat(Temporary_Quad);
NodeNumQuad = GaussStresses(:,1);

fclose(f);

%%
% Ansys does not dispose of commands that calculate the cordinate of the
% integration points in the global cordinate system, to do so, this part of
% the code was implemented to get the coordinates of theses points in the
% global system for quadrilateral, triangle and mixed mesh.

Dimensions     = 2;

% Number of side per element
NSidesElemTri  = 3;
NSidesElemQuad = 4;

% Number of GP per element
NumPtsTri   = 6;
NumPtsQuad  = 9;

Grade          = 2;
CaldN          = 0;

% Invoke pgauss function to obtain the GP coordinates in the local system 
% reference.
CoordinatesTri  = pgauss(Dimensions, NSidesElemTri, NumPtsTri);
CoordinatesQuad = pgauss(Dimensions,NSidesElemQuad,NumPtsQuad);

% Ansys provide information of gauss points in vertex nodes so it is
% suitable to keep only the gauss point coordinates close to the vertex
% nodes.
CoordinatesTri([4,5,6],:)      = [];
CoordinatesQuad([2,4,5,6,8],:) = [];

% Shape function for triangular and quadrilateral elements.
NTri = shape_f_2d(CoordinatesTri,Grade,NSidesElemTri,CaldN);
NQuad = shape_f_2d(CoordinatesQuad,Grade,NSidesElemQuad,CaldN);

NumElem       = size(topology,1);
NnodeElem     = size(topology,2);

% GP for quadrilateral elements
if size(GaussStresses,1)==NumElem*NumPtsQuad 
   XQuad = zeros(NumElem,NnodeElem);
   YQuad = zeros(NumElem,NnodeElem);

   for i=1:NumElem
       for j=1:NnodeElem
           XQuad(i,j) = Xcoord(topology(i,j));
           YQuad(i,j) = Ycoord(topology(i,j));
       end
   end
           
   GPXQuad           = XQuad*NQuad';
   GPYQuad           = YQuad*NQuad';
   TempX             = mat2cell(GPXQuad,1*ones(length(topology),1),...
                       NumPtsQuad);
   TempY             = mat2cell(GPYQuad,1*ones(length(topology),1),...
                       NumPtsQuad);
   TempX             = cellfun(@transpose,TempX,'un',0);
   TempY             = cellfun(@transpose,TempY,'un',0);
   GaussPtCoordXQuad = cell2mat(TempX);
   GaussPtCoordYQuad = cell2mat(TempY);
   GaussPtCoordXY    = [GaussPtCoordXQuad GaussPtCoordYQuad];

% GP for quadrilateral and triangular elements      
elseif (size(GaussStresses,1)< NumElem*NumPtsQuad) && ...
       (size(GaussStresses,1)> NumElem*NumPtsTri)   
    X = zeros(NumElem,NnodeElem);
    Y = zeros(NumElem,NnodeElem);
           
    for i=1:NumElem
        for j=1:NnodeElem        
        X(i,j) = Xcoord(topology(i,j));
        Y(i,j) = Ycoord(topology(i,j));  
        end
    end
    Temporary1 = mat2cell(X,1*ones(NumElem,1),NnodeElem);
    Temporary2 = mat2cell(Y,1*ones(NumElem,1),NnodeElem);
    T1         = zeros(NumElem,NnodeElem);
    T2         = zeros(NumElem,NnodeElem);
    X_tri      = zeros(NumElem,NnodeElem);
    Y_tri      = zeros(NumElem,NnodeElem);
        
    for k = 1:NumElem
        Temp1 = Temporary1{k};
        Temp2 = Temporary2{k};
        for m = 2:NnodeElem
            Tg = unique(topology(k,:)','rows');
            if size(Tg',2)==6
               X_tri(k,:)      = X(topology(k,:));
               Y_tri(k,:)      = Y(topology(k,:));
               T1              = X_tri(k,:);
               X_tri(k,:)      = X(k,:);
               Temporary1{k,1} = zeros(1,NnodeElem);
               Temporary2{k,1} = zeros(1,NnodeElem);
               T2              = Y_tri(k,:);
               Y_tri(k,:)      = Y(k,:);
            end 
        end
    end
            
    remCol1      = unique(X_tri','rows','stable');
    XtriCoord    = remCol1';
    remCol2      = unique(Y_tri','rows','stable');
    YtriCoord    = remCol2';
    GPX_tri_Mix  = XtriCoord*NTri';
    GPY_tri_Mix  = YtriCoord*NTri';
    XquadCoord   = cell2mat(Temporary1);
    YquadCoord   = cell2mat(Temporary2);
    GPXquadr_Mix = XquadCoord*NQuad';
    GPYquadr_Mix = YquadCoord*NQuad';
    GPXtri_Mix   = [GPX_tri_Mix zeros(NumElem,1)];
    GPYtri_Mix   = [GPY_tri_Mix zeros(NumElem,1)];
         
    for k = 1:NumElem
        for m = 1:NnodeElem
            if GPXquadr_Mix(k,:)==0 
                GPXquadr_Mix(k,:)=GPXtri_Mix(k,:);
            end
            if GPYquadr_Mix(k,:)==0 
                GPYquadr_Mix(k,:)=GPYtri_Mix(k,:);
            end
        end
    end
            
    TempX = mat2cell(GPXquadr_Mix,1*ones(length(topology),1),NumPtsQuad);
    TempY = mat2cell(GPYquadr_Mix,1*ones(length(topology),1),NumPtsQuad);
    TempX = cellfun(@transpose,TempX,'un',0);
    TempY = cellfun(@transpose,TempY,'un',0);
    
    GaussPtCoordX_Mix = cell2mat(TempX);
    GaussPtCoordY_Mix = cell2mat(TempY);
    GaussPtCoordX_Mix(all(GaussPtCoordX_Mix==0,2),:) = [];
    GaussPtCoordY_Mix(all(GaussPtCoordY_Mix==0,2),:) = [];
    GaussPtCoordXY    = [GaussPtCoordX_Mix GaussPtCoordY_Mix];

% GP for triangular elements      
elseif size(GaussStresses,1)==NumElem*NumPtsTri      
    Temp         = topology';
    Temp2        = unique(Temp,'stable','rows');
    topologyTemp = Temp2';
    NumElemTri   = size(topologyTemp,1);
    NnodeElemTri = size(topologyTemp,2);
    XTri         = zeros(NumElemTri,NnodeElemTri);
    YTri         = zeros(NumElemTri,NnodeElemTri);
         
    for h=1:NumElemTri
        for j=1:NnodeElemTri
            XTri(h,j) = Xcoord(topologyTemp(h,j));
            YTri(h,j) = Ycoord(topologyTemp(h,j));
        end
    end
            
    GPXTri = XTri*NTri';
    GPYTri = YTri*NTri';
    TempX  = mat2cell(GPXTri,1*ones(length(topologyTemp),1),NumPtsTri);
    TempY  = mat2cell(GPYTri,1*ones(length(topologyTemp),1),NumPtsTri);
    TempX  = cellfun(@transpose,TempX,'un',0);
    TempY  = cellfun(@transpose,TempY,'un',0);
    GaussPtCoordX  = cell2mat(TempX);
    GaussPtCoordY  = cell2mat(TempY);
    GaussPtCoordXY = [GaussPtCoordX GaussPtCoordY];
    
end
                
GaussStresses(:,1) = [];
% Matrix containing stresses and node coordinates.
FinalData = [GaussPtCoordXY GaussStresses];

%%
% This function load the data for the MLS smoothing using quadratic 
% elements

% Define the path where the data is available

FieldFEGP   = FinalData(:,3:5);
XYZGP       = FinalData(:,1:2);

NumGP      = size(XYZGP,1);
NumNodes   = size(coordinatesXY,1);
XYZout     = [ XYZGP ; coordinatesXY];

% Invoke the MLS function for quadratic element to obtain the recovered 
% stress field at integration point and nodes.
[FieldStGP_Quad]    = MLS(FieldFEGP,XYZGP,XYZout,Mesh);
FieldStGP_Quad_GP   = FieldStGP_Quad(1:NumGP,:);
FieldStGP_Quad_Node = FieldStGP_Quad(1+NumGP:end,:);

%% Prepare data to create to invoke the CreateMacroToGetMLSdata so the 
% macro file MLSresults.mac can be generated.
Quadratic_SigX  = zeros(size(topology,1),size(topology,2));
Quadratic_SigY  = zeros(size(topology,1),size(topology,2));
Quadratic_SigXY = zeros(size(topology,1),size(topology,2));

for i = 1:size(topology,1)
    for j = 1:size(topology,2)
        Quadratic_SigX(i,j)  = FieldStGP_Quad_Node(topology(i,j),1);
        Quadratic_SigY(i,j)  = FieldStGP_Quad_Node(topology(i,j),2);
        Quadratic_SigXY(i,j) = FieldStGP_Quad_Node(topology(i,j),3);
    end
end

Quadratic_SigZ  = zeros(size(topology,1),size(topology,2));
Quadratic_SigXZ = zeros(size(topology,1),size(topology,2));
Quadratic_SigYZ = zeros(size(topology,1),size(topology,2));

% Prepare the input argument for the CreateMacroToGetMLSdata function
OutputXYZ      = [coordinatesXY zeros(length(coordinatesXY),1)];
OutputTopology = unique(topology(:,1:4));
ShowUX         = zeros(length(OutputTopology),1);
ShowVY         = zeros(length(OutputTopology),1);
ShowSX         = Quadratic_SigX(:,1:4);
ShowSY         = Quadratic_SigY(:,1:4);
ShowTXY        = Quadratic_SigXY(:,1:4);
ShowSZ         = Quadratic_SigZ(:,1:4);
ShowTXZ        = Quadratic_SigXZ(:,1:4);
ShowTYZ        = Quadratic_SigYZ(:,1:4);

CreateMacroToGetMLSdata(ShowUX,ShowVY,OutputXYZ,topology(:,1:4),ShowSX,ShowSY,ShowSZ,ShowTXY,ShowTXZ,ShowTYZ)
