%**************************************************************************
% This file is part of the MLS-C Library; you can redistribute it and/or 
% modify it under the terms of the GNU Lesser General Public License as 
% published by the Free Software Foundation; either version 3 of the 
% License,or any later version.
% This program is distributed in the hope that it will be useful but 
% WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
% Public License for more details.
% You should have received a copy of the GNU Lesser General Public License
% along with GNU; see the file COPYING.  If not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
% or see <http://www.gnu.org/licenses/>.
%**************************************************************************
% Created by: 
%    E Nadal, M Ouallal, JJ Rodenas, F Chinesta and FJ Fuenmayor
%  
% Release 1.0  
% Date: 18/07/2014
%**************************************************************************
%%
function [N,dNpsi,dNeta]=shape_f_2d(Coor,Degree,NumSideElem,CaldN)
% Shape Function and its derivatives in a point for triangles and 
% quadrilaterals in local coordinates X=psi y Y=eta
%
% IMPORTANT: Node enumeration was made that way:
%            vertex node are first, then midside nodes.
%
% INPUT
    % Coor        = coordinates (X = psi e Y = eta)
    % Degree      = Element degree
    % NumSideElem = Number of side per elements (1, 3 & 4)
    % CaldN       = Derivatives of N 
    %               0    ==> no derivative
    %               else ==> derivative takes place
    
% OUTPUT
    % N           = Shape function matrix. Each row corresponds to the 
    %               shape function in a coordinate points (x,y)
    % dNpsi       = Shape function matrix derivatives with respect to psi
    % dNeta       = Shape function matrix derivatives with respect to eta

if nargin==3
    CaldN = 0;
end
    
 
X = Coor(:,1)';
Y = Coor(:,2)';

switch NumSideElem
%% TRIANGLES
%-----------
case 3 
   switch Degree
   case 1        % Linear elements
      N     = [...
               1-X-Y;...
               X;...
               Y];
      if CaldN==0
         dN = 0;
      else
         dN = [...
               -1+0.*X,-1+0.*X;...
               +1+0.*X, 0+0.*X;...
                0+0.*X, 1+0.*X];

      end
   
   case 2        % Quadratics elements
      N    =[...
            (1-X-Y).*(1-2*X-2*Y);...
            -X.*(1-2*X);...
            -Y.*(1-2*Y);...
             4*X.*(1-X-Y);...
             4*X.*Y;...
             4*Y.*(1-X-Y)];
      if CaldN==0
         dN = 0;
      else  
         dN = [...
               -3+4*X+4*Y , -3+4*X+4*Y;...
               -1+4*X     ,  0.*X        ;...
               +0.*X      , -1+4*Y    ;...
               +4-8*X-4*Y , -4*X      ;...
               +4*Y       ,  4*X      ;...
               -4*Y       , +4-4*X-8*Y];
      end
      
       otherwise  % Superior degree triangles
      disp(['Polynomial degree ' num2str(Degree)...
            'for triangles not implemented']);
      error('Stop');
   end

%%    QUADRILATERALS
case 4 
   switch Degree
       case 1     % Linears
      N = [...
          (1-Y).*(1-X)/4;...
          (1-Y).*(1+X)/4;...
          (1+Y).*(1+X)/4;...
          (1+Y).*(1-X)/4];
      if CaldN==0
         dN = 0; 
      else
         dN = [...
              (-1+Y)/4, (-1+X)/4;...
              (+1-Y)/4, (-1-X)/4;...
              (+1+Y)/4, (+1+X)/4;...
              (-1-Y)/4, (+1-X)/4];
      end
      
   case 2         % Quadratics
      N = [...
           -(1-Y).*(1-X).*(1+X+Y)/4 ;...
           -(1-Y).*(1+X).*(1-X+Y)/4 ;...
           -(1+Y).*(1+X).*(1-X-Y)/4 ;...
           -(1+Y).*(1-X).*(1+X-Y)/4 ;...
           +(1-X.^2).*(1-Y)/2       ;...
           +(1-Y.^2).*(1+X)/2       ;...
           +(1-X.^2).*(1+Y)/2       ;...
           +(1-Y.^2).*(1-X)/2       ];
      if CaldN==0
         dN = 0;
      else
         dN = [...
               -(-1+Y).*(Y+2*X)/4  , -(-1+X).*(X+2*Y)/4  ;...
               -(-1+Y).*(-Y+2*X)/4 , -(1+X).*(X-2*Y)/4   ;...
               +(Y+1).*(Y+2*X)/4   , +(1+X).*(X+2*Y)/4   ;...
               -(Y+1).*(Y-2*X)/4   , -(-1+X).*(-X+2*Y)/4 ;...
               -X.*(1-Y)           , +(-1+X.^2)/2        ;...
               +(1-Y.^2)/2         , -Y.*(1+X)           ;...
               -X.*(Y+1)           , +(1-X.^2)/2         ;...
               +(-1+Y.^2)/2        , -Y.*(1-X)           ];
      end
      
       otherwise  % Superior degree quadrilaterals
      disp([ num2str(Degree) 'Polynomial degree not programmed for quadrilaterals']);
      error('Stopping');
   end
otherwise
   disp ([num2str(NumSideElem),...
         ' Side per element showed up. it will not be used ']);
   error('Stopping');
   
end 

N  = N';
dN = dN';
  
if CaldN ~= 0
    dNpsi = dN(1:size(Coor,1),:);
    dNeta = dN(size(Coor,1)+1:end,:);
else 
    dNpsi = [];
    dNeta = [];
end
  
return;